##################################################################################################
# R code to accompany ``Quantifying sources of uncertainty in projections of future climate'' by #
# Paul Northrop and Richard Chandler                                                             #
##################################################################################################

library(arm)       # Load the arm package (also loads lme4)
library(R2WinBUGS) # Load the R2WinBUGS package

# Set current working directory to a directory containing the files 
# example.fns, superandfinite.bug and the data files 
# tempdata.Rdata and raindata.Rdata
setwd("Z:/CLIMATE/IMSC/FOR_REC")
                                                               
# input functions MCMC.half.Cauchy.finite(), two.way.sim() and REML.results()
source("examplecode.fns") 


#=================== Temperature analyses (global or regional) ======================#
# NB. the time period and region are set by which.period and which.region below.     #

load(file="tempdata.Rdata")  # load the temperature data
summary(all.temp.data)       # see which variables are present 
all.temp.data[1:10,]         # look at first 10 rows of data

which.period <- 1            # 1 for 2020-2049, 2 for 2069-2098
which.reg <- 0               # 0 for global, 1 for region 1 etc.
cat(paste("region",which.reg,", period",which.period),"\n")
cond <- which(all.temp.data$region==which.reg & all.temp.data$period==which.period) 
temp <- all.temp.data[cond,] # extract relevant data

#----------------------------------------- REML --------------------------------------#

# fit model using REML
fit.REML <- lmer(y~1+ (1|scenario)+(1|GCM)+(1|factor(scenario):factor(GCM)), data=temp, REML=T) 

# no confidence intervals (CI=F)
REML.res <- REML.results(fit.REML,x1=temp$GCM,x2=temp$scenario,CI=F)
REML.res

# conf% confidence intervals (CI=T, n.sim simulations, my.seed=(optional) random number seed)
# The simulation number is printed to the screen (every 100 iterations).
# With n.sim=1000 this will take a few minutes
REML.res <- REML.results(fit.REML,x1=temp$GCM,x2=temp$scenario,CI=T,conf=95,n.sim=1000,my.seed=1)
REML.res

#----------------------------------- Bayesian analysis -------------------------------#

if (which.period==1) my.A <- 0.5 # set scale parameter of half-Cauchy prior for SDs
if (which.period==2) my.A <- 1

# (A WinBUGS window will open and close when the calculations finish.)
# The underlying function is bugs().  Use ...
?bugs
# to see what its arguments mean and what its output are.

# Set my.bugs as the location in which WinBUGS is installed on your computer
# (especially if this location is not "C:/Program Files/WinBUGS14/"

my.bugs <- "C:/Users/paul/Documents/WinBUGS14"

#----------  Super-population and finite-population standard devations -------------#

set.seed(200)  
my.sim.fin <- MCMC.half.Cauchy.finite(temp,A=my.A,n.iter=2500,n.chains=5,n.burnin=500,
   n.thin=1,bugs.directory=my.bugs)
Bayes.res.fin <- my.sim.fin$summary[-10,]
row.names(Bayes.res.fin) <- c("mu","sigma_G","sigma_S","sigma_GS","sigma_R",
   "s_G","s_S","s_GS","s_R")
print(round(Bayes.res.fin,3)) # summary of the results
summary(my.sim.fin)           # my.sim contains lots of other stuff!

#=================== Precipitation analyses (global or regional) ====================#

load(file="raindata.Rdata") # load the rainfall data
summary(all.rain.data)

which.period <- 1          # 1 for 2020-2049, 2 for 2069-2098
which.reg <- 0             # 0 for global, 1 for region 1 etc.
cat(paste("region",which.reg,", period",which.period),"\n")
cond <- which(all.rain.data$region==which.reg & all.rain.data$period==which.period) 
rain <- all.rain.data[cond,]                                      # extract relevant data

#----------------------------------- Bayesian analysis -------------------------------#

if (which.period==1) my.A <- 2.5 # set scale parameter of half-Cauchy prior for SDs
if (which.period==2) my.A <- 5

#----------  Super-population and finite-population standard devations -------------#

set.seed(200)              
my.sim.fin.prec <- MCMC.half.Cauchy.finite(rain,A=my.A,n.iter=2500,n.chains=5,n.burnin=500,
   n.thin=1,bugs.directory=my.bugs)
Bayes.prec.res.fin <- my.sim.fin.prec$summary[-10,]
row.names(Bayes.prec.res.fin) <- c("mu","sigma_G","sigma_S","sigma_GS","sigma_R",
   "s_G","s_S","s_GS","s_R")
print(round(Bayes.prec.res.fin,3)) # summary of the results





